"{ Package: 'stx:libcomp' }"

ProgramNodeEnumerator subclass:#ParseNodeValidator
	instanceVariableNames:'nodeStack'
	classVariableNames:''
	poolDictionaries:''
	category:'System-Compiler-Support'
!

!ParseNodeValidator class methodsFor:'documentation'!

documentation
"
    A helper class used to validate parse tree, i.e. parent instvar,
    startPosition/endPositions. Useful only for Parser debugging/hacking.

    [author:]
        Jan Vrany <jan.vrany@fit.cvut.cz>

    [instance variables:]

    [class variables:]

    [see also:]

"
! !

!ParseNodeValidator class methodsFor:'validation'!

validate: aParseNode

    ^self basicNew validate: aParseNode

    "Created: / 20-07-2011 / 20:31:43 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

validateImage

    self validateImage: false.

    "Created: / 20-07-2011 / 20:41:34 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

validateImage: stopOnError

    Smalltalk allClassesDo:[:cls|
        cls isLoaded ifTrue:[
            Transcript nextPut:$..
            cls methodsDo:[:mth|
                [ 
                    self validate: (Parser parseMethod: mth source) tree 
                ] on: Error do:[:ex|
                    Transcript 
                        cr;
                        show: mth printString;
                        show: '...FAILED!!';
                        cr.                        
                    stopOnError ifTrue:[
                        ex pass
                    ].
                ]                
            ].
        ].
    ].

    "Created: / 20-07-2011 / 20:42:32 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!ParseNodeValidator methodsFor:'enumeration callbacks'!

doMethod:aNode selector:sel primitive:prim block:block

    self doNode: aNode.

    "Modified: / 20-07-2011 / 20:29:14 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

doParameter:aNode variable:var type:type

    self doNode: aNode.

    "Modified: / 20-07-2011 / 20:29:29 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

doSequence:aNode temporaries:temps statements:stats

    self doNode: aNode expression.    
    aNode nextStatement notNil ifTrue:[
        nodeStack pop.
        self doNode: aNode nextStatement.
        nodeStack push: aNode nextStatement.
    ].

    "Created: / 20-07-2011 / 21:46:54 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!ParseNodeValidator methodsFor:'private'!

doNode:aNode

    self assert: aNode parent == nodeStack top 
         description: 'each node should have parent'.

    nodeStack push: aNode.
    super doNode: aNode.
    nodeStack pop.

    "Created: / 20-07-2011 / 20:36:58 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!ParseNodeValidator methodsFor:'validation'!

validate: tree

    nodeStack := Stack with: nil.

    ^self doNode: tree

    "Created: / 20-07-2011 / 20:31:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!ParseNodeValidator class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libcomp/ParseNodeValidator.st,v 1.1 2011-07-20 21:08:23 vrany Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/libcomp/ParseNodeValidator.st,v 1.1 2011-07-20 21:08:23 vrany Exp $'
! !
